=== modified file 'configure.ac'
--- configure.ac	2010-09-17 23:30:41 +0000
+++ configure.ac	2010-09-21 09:57:03 +0000
@@ -81,6 +81,8 @@
 PANDORA_REQUIRE_LIBDL
 AC_LANG_POP
 
+DRIZZLE_STACK_DIRECTION
+
 PANDORA_USE_BETTER_MALLOC
 
 PANDORA_DRIZZLE_BUILD

=== modified file 'drizzled/internal/my_thr_init.cc'
--- drizzled/internal/my_thr_init.cc	2010-09-19 23:55:55 +0000
+++ drizzled/internal/my_thr_init.cc	2010-09-21 09:57:03 +0000
@@ -39,12 +39,16 @@
 # endif
 #endif
 
+#include <boost/thread/thread.hpp>
+#include <boost/thread/mutex.hpp>
+#include <boost/thread/tss.hpp>
+
 namespace drizzled
 {
 namespace internal
 {
 
-pthread_key_t THR_KEY_mysys;
+boost::thread_specific_ptr<st_my_thread_var> THR_KEY_mysys;
 boost::mutex THR_LOCK_threads;
 pthread_cond_t  THR_COND_threads;
 uint32_t THR_thread_count= 0;
@@ -63,14 +67,6 @@
 
 bool my_thread_global_init(void)
 {
-  int pth_ret;
-
-  if ((pth_ret= pthread_key_create(&THR_KEY_mysys, NULL)) != 0)
-  {
-    fprintf(stderr,"Can't initialize threads: error %d\n", pth_ret);
-    return 1;
-  }
-
   pthread_cond_init(&THR_COND_threads, NULL);
   if (my_thread_init())
   {
@@ -110,7 +106,6 @@
     }
   }
 
-  pthread_key_delete(THR_KEY_mysys);
   if (all_threads_killed)
   {
     pthread_cond_destroy(&THR_COND_threads);
@@ -136,19 +131,15 @@
   st_my_thread_var *tmp= NULL;
 
   // We should mever see my_thread_init()  called twice
-  if (pthread_getspecific(THR_KEY_mysys))
+  if (THR_KEY_mysys.get())
     return 0;
 
-  tmp= static_cast<st_my_thread_var *>(calloc(1, sizeof(*tmp)));
+  tmp= new st_my_thread_var;
   if (tmp == NULL)
   {
     return 1;
   }
-  pthread_setspecific(THR_KEY_mysys,tmp);
-  tmp->pthread_self= pthread_self();
-  pthread_mutex_init(&tmp->mutex,MY_MUTEX_INIT_FAST);
-  pthread_cond_init(&tmp->suspend, NULL);
-  tmp->init= 1;
+  THR_KEY_mysys.reset(tmp);
 
   boost::mutex::scoped_lock scopedLock(THR_LOCK_threads);
   tmp->id= ++thread_id;
@@ -172,17 +163,12 @@
 
 void my_thread_end(void)
 {
-  st_my_thread_var *tmp=
-    static_cast<st_my_thread_var *>(pthread_getspecific(THR_KEY_mysys));
+  st_my_thread_var *tmp= THR_KEY_mysys.get();
 
-  if (tmp && tmp->init)
+  if (tmp)
   {
-#if !defined(__bsdi__) && !defined(__OpenBSD__)
- /* bsdi and openbsd 3.5 dumps core here */
-    pthread_cond_destroy(&tmp->suspend);
-#endif
-    pthread_mutex_destroy(&tmp->mutex);
-    free(tmp);
+    delete tmp;
+    THR_KEY_mysys.release();
 
     /*
       Decrement counter for number of running threads. We are using this
@@ -199,8 +185,7 @@
 
 struct st_my_thread_var *_my_thread_var(void)
 {
-  struct st_my_thread_var *tmp= (struct st_my_thread_var*)pthread_getspecific(THR_KEY_mysys);
-  return tmp;
+  return THR_KEY_mysys.get();
 }
 
 } /* namespace internal */

=== modified file 'drizzled/internal/thread_var.h'
--- drizzled/internal/thread_var.h	2010-08-07 22:01:28 +0000
+++ drizzled/internal/thread_var.h	2010-09-21 09:57:03 +0000
@@ -28,6 +28,11 @@
 namespace internal
 {
 
+static pthread_t wrapper_pthread_self()
+{
+  return pthread_self();
+}
+
 struct st_my_thread_var
 {
   pthread_cond_t suspend;
@@ -37,9 +42,28 @@
   pthread_t pthread_self;
   uint64_t id;
   int volatile abort;
-  bool init;
   struct st_my_thread_var *next,**prev;
   void *opt_info;
+
+  st_my_thread_var() :
+    current_mutex(0),
+    current_cond(0),
+    id(0),
+    abort(false),
+    next(0),
+    prev(0),
+    opt_info(0)
+  { 
+    pthread_self= wrapper_pthread_self();
+    pthread_mutex_init(&mutex, NULL);
+    pthread_cond_init(&suspend, NULL);
+  }
+
+  ~st_my_thread_var()
+  {
+    pthread_cond_destroy(&suspend);
+    pthread_mutex_destroy(&mutex);
+  }
 };
 
 extern struct st_my_thread_var *_my_thread_var(void);

=== modified file 'drizzled/session.cc'
--- drizzled/session.cc	2010-09-20 00:00:20 +0000
+++ drizzled/session.cc	2010-09-21 09:57:03 +0000
@@ -464,7 +464,6 @@
     This allows us to move Session to different threads if needed.
   */
   mysys_var->id= thread_id;
-  real_id= pthread_self();                      // For debugging
 
   /*
     We have to call thr_lock_info_init() again here as Session may have been

=== modified file 'drizzled/session.h'
--- drizzled/session.h	2010-09-19 22:18:23 +0000
+++ drizzled/session.h	2010-09-21 09:57:03 +0000
@@ -678,7 +678,6 @@
     create_sort_index(); may differ from examined_row_count.
   */
   uint32_t row_count;
-  pthread_t real_id; /**< For debugging */
   uint64_t thread_id;
   uint32_t tmp_table;
   uint32_t global_read_lock;

=== added file 'm4/stack.m4'
--- m4/stack.m4	1970-01-01 00:00:00 +0000
+++ m4/stack.m4	2010-09-21 09:57:03 +0000
@@ -0,0 +1,36 @@
+AC_DEFUN([DRIZZLE_STACK_DIRECTION],[
+  AC_CACHE_CHECK(stack direction for C alloca, ac_cv_c_stack_direction,
+    [AC_RUN_IFELSE(
+       [AC_LANG_PROGRAM(
+          [[
+	    #include <stdlib.h>
+
+ /* Prevent compiler optimization by HP's compiler, see bug#42213 */
+ #if defined(__HP_cc) || defined (__HP_aCC) || defined (__hpux)
+ #pragma noinline
+ #endif
+
+ int find_stack_direction ()
+ {
+   static char *addr = 0;
+   auto char dummy;
+   if (addr == 0)
+     {
+       addr = &dummy;
+       return find_stack_direction ();
+     }
+   else
+     return (&dummy > addr) ? 1 : -1;
+ }
+	  ]],
+	  [[
+   exit (find_stack_direction() < 0);
+	  ]]
+       )],
+       [ac_cv_c_stack_direction=1],
+       [ac_cv_c_stack_direction=-1],
+       [ac_cv_c_stack_direction=]
+    )]
+  )
+ AC_DEFINE_UNQUOTED(STACK_DIRECTION, $ac_cv_c_stack_direction)
+])

=== modified file 'plugin/multi_thread/multi_thread.cc'
--- plugin/multi_thread/multi_thread.cc	2010-09-14 01:11:01 +0000
+++ plugin/multi_thread/multi_thread.cc	2010-09-21 09:57:03 +0000
@@ -20,6 +20,9 @@
 #include <drizzled/module/option_map.h>
 #include <drizzled/errmsg_print.h>
 
+#include <boost/thread.hpp>
+#include <boost/bind.hpp>
+
 namespace po= boost::program_options;
 using namespace std;
 using namespace drizzled;
@@ -35,31 +38,42 @@
   extern size_t my_thread_stack_size;
 }
 
-/**
- * Function to be run as a thread for each session.
- */
-namespace
-{
-  extern "C" pthread_handler_t session_thread(void *arg);
-}
-
-namespace
-{
-  extern "C" pthread_handler_t session_thread(void *arg)
-  {
-    Session *session= static_cast<Session*>(arg);
-    MultiThreadScheduler *sched= static_cast<MultiThreadScheduler*>(session->scheduler);
-    sched->runSession(session);
-    return NULL;
-  }
-}
-
-
-bool MultiThreadScheduler::addSession(Session *session)
-{
-  if (thread_count >= max_threads)
-    return true;
-
+void MultiThreadScheduler::runSession(drizzled::Session *session)
+{
+  if (drizzled::internal::my_thread_init())
+  {
+    session->disconnect(drizzled::ER_OUT_OF_RESOURCES, true);
+    session->status_var.aborted_connects++;
+    killSessionNow(session);
+  }
+  boost::this_thread::at_thread_exit(&internal::my_thread_end);
+
+  session->thread_stack= (char*) &session;
+  session->run();
+  killSessionNow(session);
+}
+
+void MultiThreadScheduler::setStackSize()
+{
+  pthread_attr_t attr;
+
+  (void) pthread_attr_init(&attr);
+
+  /* Get the thread stack size that the OS will use and make sure
+    that we update our global variable. */
+  int err= pthread_attr_getstacksize(&attr, &my_thread_stack_size);
+  pthread_attr_destroy(&attr);
+
+  if (err != 0)
+  {
+    errmsg_printf(ERRMSG_LVL_ERROR, _("Unable to get thread stack size\n"));
+    my_thread_stack_size= 524288; // At the time of the writing of this code, this was OSX's
+  }
+
+  if (my_thread_stack_size == 0)
+  {
+    my_thread_stack_size= 524288; // At the time of the writing of this code, this was OSX's
+  }
 #ifdef __sun
   /*
    * Solaris will return zero for the stack size in a call to
@@ -71,39 +85,22 @@
    * will be used.
    */
   if (my_thread_stack_size == 0)
+  {
     my_thread_stack_size= 2 * 1024 * 1024;
+  }
 #endif
-
-  /* Thread stack size of zero means just use the OS default */
-  if (my_thread_stack_size != 0)
-  {
-    int err= pthread_attr_setstacksize(&attr, my_thread_stack_size);
-
-    if (err != 0)
-    {
-      errmsg_printf(ERRMSG_LVL_ERROR,
-                    _("Unable to set thread stack size to %" PRId64 "\n"),
-                    static_cast<uint64_t>(my_thread_stack_size));
-      return true;
-    }
-  }
-  else
-  {
-    /* Get the thread stack size that the OS will use and make sure
-       that we update our global variable. */
-    int err= pthread_attr_getstacksize(&attr, &my_thread_stack_size);
-
-    if (err != 0)
-    {
-      errmsg_printf(ERRMSG_LVL_ERROR, _("Unable to get thread stack size\n"));
-      return true;
-    }
-  }
+}
+
+bool MultiThreadScheduler::addSession(Session *session)
+{
+  if (thread_count >= max_threads)
+    return true;
 
   thread_count.increment();
 
-  if (pthread_create(&session->real_id, &attr, session_thread,
-                     static_cast<void*>(session)))
+  boost::thread new_thread(boost::bind(&MultiThreadScheduler::runSession, this, session));
+
+  if (not new_thread.joinable())
   {
     thread_count.decrement();
     return true;
@@ -118,9 +115,6 @@
   /* Locks LOCK_thread_count and deletes session */
   Session::unlink(session);
   thread_count.decrement();
-  internal::my_thread_end();
-  pthread_exit(0);
-  /* We should never reach this point. */
 }
 
 MultiThreadScheduler::~MultiThreadScheduler()
@@ -130,8 +124,6 @@
   {
     COND_thread_count.wait(scopedLock);
   }
-
-  (void) pthread_attr_destroy(&attr);
 }
 
   

=== modified file 'plugin/multi_thread/multi_thread.h'
--- plugin/multi_thread/multi_thread.h	2010-09-14 06:43:02 +0000
+++ plugin/multi_thread/multi_thread.h	2010-09-21 09:57:03 +0000
@@ -29,17 +29,12 @@
 {
 private:
   drizzled::atomic<uint32_t> thread_count;
-  pthread_attr_t attr;
 
 public:
   MultiThreadScheduler(const char *name_arg): 
     Scheduler(name_arg)
   {
-    /* Setup attribute parameter for session threads. */
-    (void) pthread_attr_init(&attr);
-    (void) pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-    pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);
-
+    setStackSize();
     thread_count= 0;
   }
 
@@ -47,19 +42,9 @@
   bool addSession(drizzled::Session *session);
   void killSessionNow(drizzled::Session *session);
   
-  void runSession(drizzled::Session *session)
-  {
-    if (drizzled::internal::my_thread_init())
-    {
-      session->disconnect(drizzled::ER_OUT_OF_RESOURCES, true);
-      session->status_var.aborted_connects++;
-      killSessionNow(session);
-    }
-
-    session->thread_stack= (char*) &session;
-    session->run();
-    killSessionNow(session);
-  }
+  void runSession(drizzled::Session *session);
+private:
+  void setStackSize();
 };
 
 #endif /* PLUGIN_MULTI_THREAD_MULTI_THREAD_H */

